from .command import *
from .core import *
from . import command


class Request:
    id = 0
    _tx = Packet(b'', False, 0)

    class RequestItem:
        def __init__(self, request):
            self._rq = request
            self._pos = 0
            self. _limit = min(len(request._cmds), len(request._vals))

        def __iter__(self):
            self._pos = 0
            return self

        def __next__(self):
            if self._pos >= self._limit:
                raise StopIteration
            else:
                res = self._rq._cmds[self._pos], self._rq._vals[self._pos]
                self._pos += 1
                return res

    def __init__(self):
        self._cmds = []
        self._vals = []

    def set_answer(self, data):
        p = Packet(data, 'auto')
        hdr = p.read_byte()
        if hdr != self.id:
            raise SmpException()
        self.ipf = p.read_byte() & 0x40 != 0
        self._rx = p

    def get_request(self):
        return self._tx.data

    def commands(self):
        return self._cmds

    def values(self):
        return self._vals

    def items(self):
        return Request.RequestItem(self)

    def decode(self):
        pass

    def encode(self):
        dff = False
        for cmd in self._cmds:
            if cmd.require_dff(): dff = True
        p = Packet(b'', dff, 0)
        p.write_byte(self.id)
        p.write_byte(0x80 if dff else 0)
        for cmd in self._cmds:
            p.write_param(cmd.code)
            cmd.encode_param(p)


        self._tx = p


class Action(Request):
    id = PERFORM_ACTION
    def add(self, code, value):
        self._cmds.append(CommandAction(code, value))


class GetSet(Request):
    id = GETSET
    _cmd_maker = {
        GET_DATA_SINGLE: command.CommandDataSingle,
        GET_DATA_MULTIPLE: command.CommandDataMultiple,
        GET_OPTION: command.CommandGetOption,
        SET_OPTION: command.CommandSetOption,
        GET_INFO: command.CommandInfo,
        PERFORM_ACTION: command.CommandAction,
        GET_OPTION1: command.CommandGetOption,
        GET_DATA_SINGLE_EX: command.CommandDataSingleEx,
        GET_DATA_MULTIPLE_EX: command.CommandDataMultipleEx,
        }

    def __init__(self):
        super().__init__()

    def decode(self):
        rx = self._rx
        while not rx.eof():
            code_class = rx.read_param()
            while not rx.eof():
                code = rx.read_param()
                if code == 0: # error
                    code = rx.read_param()
                    if code == 0: break # end of code class

                cmd = self._cmd_maker[code_class](code)
                cmd.decode_param(rx)
                if len(self._cmds) <= len(self._vals):
                    self._cmds.append(cmd)
                else:
                    if self._cmds[len(self._vals)] != cmd:
                        raise SmpException()
                self._vals.append(cmd.decode_value(rx))



#коды DATA_SINGLE
SMP_SDATA_CURR_NRG_APOS_CODE = 1
SMP_SDATA_CURR_NRG_ANEG_CODE = 2
SMP_SDATA_CURR_NRG_RPOS_CODE = 3
SMP_SDATA_CURR_NRG_RNEG_CODE = 4
SMP_SDATA_FIXED_NRG_APOS_CODE = 5
SMP_SDATA_FIXED_NRG_ANEG_CODE = 6
SMP_SDATA_FIXED_NRG_RPOS_CODE = 7
SMP_SDATA_FIXED_NRG_RNEG_CODE = 8
SMP_SDATA_TIMEBACK_NRG_APOS_CODE = 9
SMP_SDATA_TIMEBACK_NRG_ANEG_CODE = 10
SMP_SDATA_TIMEBACK_NRG_RPOS_CODE = 11
SMP_SDATA_TIMEBACK_NRG_RNEG_CODE = 12
SMP_SDATA_CURR_POW_FULL_CODE = 13
SMP_SDATA_CURR_POW_A_CODE = 14
SMP_SDATA_CURR_POW_R_CODE = 16
SMP_SDATA_HALFHOUR_POW_A_CODE = 18
SMP_SDATA_HALFHOUR_POW_ANEG_CODE = 19
SMP_SDATA_HALFHOUR_POW_RPOS_CODE = 20
SMP_SDATA_HALFHOUR_POW_RNEG_CODE = 21
SMP_SDATA_LINEAR_CURRENT_CODE = 22
SMP_SDATA_NEUTRAL_CURRENT_CODE = 23
SMP_SDATA_VOLTAGE_CODE = 24
SMP_SDATA_COS_PHI_CODE = 25
SMP_SDATA_FREQUENCY_CODE = 26
SMP_SDATA_LAST_DIP_DEPTH_CODE = 27
SMP_SDATA_LAST_DIP_TIME_CODE = 28
SMP_SDATA_OVERVOLTAGE_VALUE_CODE = 29
SMP_SDATA_OVERVOLTAGE_TIME_CODE = 30
SMP_SDATA_TEMPERATURE_CODE = 31
SMP_SDATA_BATTERY_CODE = 32
SMP_SDATA_TMR_MAGNET_CODE = 33
SMP_SDATA_TMR_POW_LACK_CODE = 34
SMP_SDATA_TMR_POW_LOW_CODE = 35
SMP_SDATA_TMR_POW_HIGH_CODE = 36
SMP_SDATA_TMR_POW_OVERLIM_CODE = 37
SMP_SDATA_TMR_DIFF_OVERLIM_CODE = 38
SMP_SDATA_TMR_FREQ_MISMATCH_CODE = 39
SMP_SDATA_TMR_EXCHANGE_LACK_CODE = 40
SMP_SDATA_TMR_CONSUMP_LOW_CODE = 41
SMP_SDATA_TMR_MISTIME_CODE = 42
SMP_SDATA_TMR_DELTATIME_CODE = 44
SMP_SDATA_CURR_NRG_APOS_PULSES_CODE = 45
SMP_SDATA_CURR_NRG_ANEG_PULSES_CODE = 46
SMP_SDATA_CURR_NRG_RPOS_PULSES_CODE = 47
SMP_SDATA_CURR_NRG_RNEG_PULSES_CODE = 48
SMP_SDATA_DATETIME_CODE = 49
SMP_SDATA_SERIAL_NUM_CODE = 50
SMP_SDATA_SUBSCRIBER_NUM_CODE = 51
SMP_SDATA_TERM_TYPE_CODE = 52
SMP_SDATA_DATA_TYPE_CODE = 53
SMP_SDATA_CURR_NRG_GROUP_CODE = 54
SMP_SDATA_CURR_NRG_GROUP_PULSES_CODE = 55
SMP_SDATA_CURR_POW_FULL_CANAL_CODE = 56
SMP_SDATA_CURR_POW_A_CANAL_CODE = 57
SMP_SDATA_CURR_POW_R_CANAL_CODE = 58
SMP_SDATA_VOLTAGE_CANAL_CODE = 59
SMP_SDATA_CURRENT_CANAL_CODE = 60
SMP_SDATA_COS_PHI_CANAL_CODE = 61
SMP_SDATA_FREQUENCY_CANAL_CODE = 62
SMP_SDATA_3PH_CALIBR_PC_CODE = 63
SMP_SDATA_3PH_CALIBR_IGAIN_CODE = 64
SMP_SDATA_3PH_CALIBR_VGAIN_CODE = 65
SMP_SDATA_3PH_CALIBR_POFF_CODE = 66
SMP_SDATA_3PH_CALIBR_QOFF_CODE = 67
SMP_SDATA_3PH_CALIBR_IACOFF_CODE = 68
SMP_SDATA_3PH_CALIBR_REGCHK_CODE = 69
SMP_SDATA_3PH_CALIBR_TIME_CODE = 70
SMP_SDATA_TMR_ERROR_ORDER_FAZ_CODE = 71
SMP_SDATA_TMR_OVER_RADIO_COODE = 72
SMP_SDATA_CALIBR_N_PC_CODE = 73
SMP_SDATA_CALIBR_N_IGAIN_CODE = 74
SMP_SDATA_CALIBR_N_VGAIN_CODE = 75
SMP_SDATA_CALIBR_N_POFF_CODE = 76
SMP_SDATA_CALIBR_N_QOFF_CODE = 77
SMP_SDATA_CALIBR_N_IACOFF_CODE = 78
SMP_SDATA_CALIBR_N_REGCHK_CODE = 79
SMP_SDATA_CALIBR_N_TIME_CODE = 80
SMP_SDATA_TMR_OVER_VAR_MAGNET_COODE = 81
SMP_SDATA_ANGLE_CODE = 82
SMP_SDATA_PHASE_ANGLE_CODE = 83
SMP_SDATA_LINEAR_VOLTAGE_CODE = 84
SMP_SDATA_NEUTRAL_CALC_CURRENT_CODE = 85
SMP_SDATA_CALIBRATION_DATA_CODE = 86

SMP_SDATA_PREDICT_POW_APOS_CODE = 98
SMP_SDATA_PREDICT_POW_ANEG_CODE = 99
SMP_SDATA_PREDICT_POW_RPOS_CODE = 100
SMP_SDATA_PREDICT_POW_RNEG_CODE = 101
SMP_SDATA_COUNT_EXT1_CODE = 102
SMP_SDATA_COUNT_EXT2_CODE = 103

SMP_SDATA_COUNT_CH1_CODE = 110
SMP_SDATA_COUNT_CH2_CODE = 111

#коды DATA_MULTIPLE
SMP_MDATA_DAILY_NRG_APOS_CODE = 1
SMP_MDATA_DAILY_NRG_ANEG_CODE = 2
SMP_MDATA_DAILY_NRG_RPOS_CODE = 3
SMP_MDATA_DAILY_NRG_RNEG_CODE = 4
SMP_MDATA_DAY_NRG_APOS_CODE = 5
SMP_MDATA_DAY_NRG_ANEG_CODE = 6
SMP_MDATA_DAY_NRG_RPOS_CODE = 7
SMP_MDATA_DAY_NRG_RNEG_CODE = 8
SMP_MDATA_MONTHLY_NRG_APOS_CODE = 9
SMP_MDATA_MONTHLY_NRG_ANEG_CODE = 10
SMP_MDATA_MONTHLY_NRG_RPOS_CODE = 11
SMP_MDATA_MONTHLY_NRG_RNEG_CODE = 12
SMP_MDATA_MONTH_NRG_APOS_CODE = 13
SMP_MDATA_MONTH_NRG_ANEG_CODE = 14
SMP_MDATA_MONTH_NRG_RPOS_CODE = 15
SMP_MDATA_MONTH_NRG_RNEG_CODE = 16
SMP_MDATA_YEARLY_NRG_APOS_CODE = 17
SMP_MDATA_YEARLY_NRG_ANEG_CODE = 18
SMP_MDATA_YEARLY_NRG_RPOS_CODE = 19
SMP_MDATA_YEARLY_NRG_RNEG_CODE = 20
SMP_MDATA_YEAR_NRG_APOS_CODE = 21
SMP_MDATA_YEAR_NRG_ANEG_CODE = 22
SMP_MDATA_YEAR_NRG_RPOS_CODE = 23
SMP_MDATA_YEAR_NRG_RNEG_CODE = 24
SMP_MDATA_PROF_NRG_APOS_CODE = 25
SMP_MDATA_PROF_NRG_ANEG_CODE = 26
SMP_MDATA_PROF_NRG_RPOS_CODE = 27
SMP_MDATA_PROF_NRG_RNEG_CODE = 28
SMP_MDATA_25H_PROF_NRG_APOS_CODE = 29
SMP_MDATA_25H_PROF_NRG_ANEG_CODE = 30
SMP_MDATA_25H_PROF_NRG_RPOS_CODE = 31
SMP_MDATA_25H_PROF_NRG_RNEG_CODE = 32
SMP_MDATA_PROF_VOLTAGE_CODE = 33
SMP_MDATA_PROF_FREQUENCY_CODE = 34
SMP_MDATA_MAX_POW_A_CODE = 35
SMP_MDATA_MAX_POW_A_DATE_CODE = 36
SMP_MDATA_DAILY_NRG_APOS_PULSES_CODE = 37
SMP_MDATA_DAILY_NRG_ANEG_PULSES_CODE = 38
SMP_MDATA_DAILY_NRG_RPOS_PULSES_CODE = 39
SMP_MDATA_DAILY_NRG_RNEG_PULSES_CODE = 40
SMP_MDATA_MONTHLY_NRG_APOS_PULSES_CODE = 41
SMP_MDATA_MONTHLY_NRG_ANEG_PULSES_CODE = 42
SMP_MDATA_MONTHLY_NRG_RPOS_PULSES_CODE = 43
SMP_MDATA_MONTHLY_NRG_RNEG_PULSES_CODE = 44
SMP_MDATA_PROF_NRG_APOS_PULSES_CODE = 45
SMP_MDATA_PROF_NRG_ANEG_PULSES_CODE = 46
SMP_MDATA_PROF_NRG_RPOS_PULSES_CODE = 47
SMP_MDATA_PROF_NRG_RNEG_PULSES_CODE = 48
SMP_MDATA_DAILY_NRG_GROUP_CODE = 49
SMP_MDATA_MONTHLY_NRG_GROUP_CODE = 50
SMP_MDATA_PROF_NRG_GROUP_CODE = 51
SMP_MDATA_DAILY_NRG_GROUP_PULSES_CODE = 52
SMP_MDATA_MONTHLY_NRG_GROUP_PULSES_CODE = 53
SMP_MDATA_PROF_NRG_GROUP_PULSES_CODE = 54
SMP_MDATA_25H_PROF_VOLTAGE_CODE = 55
SMP_MDATA_25H_PROF_FREQUENCY_CODE = 56
SMP_MDATA_PROF_CURRENT_CODE = 57
SMP_MDATA_25H_PROF_CURRENT_CODE = 58
SMP_MDATA_PROF_BANK3_CODE = 59
SMP_MDATA_PROF_BANK4_CODE = 60
SMP_MDATA_25H_PROF_BANK3_CODE = 61
SMP_MDATA_25H_PROF_BANK4_CODE = 62
SMP_MDATA_PROF_POW_S_CODE = 63
SMP_MDATA_PROF_POW_A_CODE = 64
SMP_MDATA_PROF_POW_R_CODE = 65
SMP_MDATA_PROF_POW_PF_CODE = 66

SMP_MDATA_EVLOG_FULL_CODE = 200
SMP_MDATA_EVLOG_BYNO_CODE = 201
SMP_MDATA_EVENT_APOS_TIME_CODE = 202
SMP_MDATA_EVENT_ANEG_TIME_CODE = 203
SMP_MDATA_EVENT_RPOS_TIME_CODE = 204
SMP_MDATA_EVENT_RNEG_TIME_CODE = 205
SMP_MDATA_EVLOG_FULL_SEC_CODE = 206
SMP_MDATA_EVLOG_BYNO_SEC_CODE = 207
SMP_MDATA_EVENT_APOS_CODE = 208
SMP_MDATA_EVENT_ANEG_CODE = 209
SMP_MDATA_EVENT_RPOS_CODE = 210
SMP_MDATA_EVENT_RNEG_CODE = 211

SMP_MDATA_PROF_BY_NUM_1_CODE = 213
SMP_MDATA_PROF_BY_NUM_2_CODE = 214
SMP_MDATA_PROF_BY_NUM_3_CODE = 215
SMP_MDATA_PROF_BY_NUM_4_CODE = 216
SMP_MDATA_PROF_BY_NUM_5_CODE = 217
SMP_MDATA_PROF_BY_NUM_6_CODE = 218
SMP_MDATA_PROF_BY_NUM_7_CODE = 219
SMP_MDATA_PROF_BY_NUM_8_CODE = 220
SMP_MDATA_PROF_BY_NUM_9_CODE = 221
SMP_MDATA_PROF_BY_NUM_10_CODE = 222
SMP_MDATA_PROF_BY_NUM_11_CODE = 223
SMP_MDATA_PROF_BY_NUM_12_CODE = 224
SMP_MDATA_PROF_BY_NUM_13_CODE = 225
SMP_MDATA_PROF_BY_NUM_14_CODE = 226
SMP_MDATA_PROF_BY_NUM_15_CODE = 227
SMP_MDATA_PROF_BY_NUM_16_CODE = 228
SMP_MDATA_PROF_BY_NUM_17_CODE = 229
SMP_MDATA_PROF_BY_NUM_18_CODE = 230
SMP_MDATA_PROF_BY_NUM_19_CODE = 231
SMP_MDATA_PROF_BY_NUM_20_CODE = 232
SMP_MDATA_PROF_BY_NUM_21_CODE = 233
SMP_MDATA_PROF_BY_NUM_22_CODE = 234
SMP_MDATA_PROF_BY_NUM_23_CODE = 235
SMP_MDATA_PROF_BY_NUM_24_CODE = 236

#коды OPTION
SMP_OPT_TIME_DATE_CODE = 1
SMP_OPT_TIME_CORR_CODE = 2
SMP_OPT_DST_MODE_CODE = 3
SMP_OPT_DST_OFF_TIME_CODE = 4
SMP_OPT_DST_ON_TIME_CODE = 5
SMP_OPT_MAX_TIME_DIFFERENCE_CODE = 6
SMP_OPT_MAX_MISTIME_CODE = 7
SMP_OPT_TIME_CTRL_CODE = 8
SMP_OPT_EVENT_GROUPS_CODE = 9
SMP_OPT_TARIFICATION_MODES_CODE = 10
SMP_OPT_SHED_DAILY_01_CODE = 11
SMP_OPT_SHED_DAILY_02_CODE = 12
SMP_OPT_SHED_DAILY_03_CODE = 13
SMP_OPT_SHED_DAILY_04_CODE = 14
SMP_OPT_SHED_DAILY_05_CODE = 15
SMP_OPT_SHED_DAILY_06_CODE = 16
SMP_OPT_SHED_DAILY_07_CODE = 17
SMP_OPT_SHED_DAILY_08_CODE = 18
SMP_OPT_SHED_DAILY_09_CODE = 19
SMP_OPT_SHED_DAILY_10_CODE = 20
SMP_OPT_SHED_DAILY_11_CODE = 21
SMP_OPT_SHED_DAILY_12_CODE = 22
SMP_OPT_SHED_DAILY_13_CODE = 23
SMP_OPT_SHED_DAILY_14_CODE = 24
SMP_OPT_SHED_DAILY_15_CODE = 25
SMP_OPT_SHED_DAILY_16_CODE = 26
SMP_OPT_SHED_DAILY_17_CODE = 27
SMP_OPT_SHED_DAILY_18_CODE = 28
SMP_OPT_SHED_DAILY_19_CODE = 29
SMP_OPT_SHED_DAILY_20_CODE = 30
SMP_OPT_SHED_DAILY_21_CODE = 31
SMP_OPT_SHED_DAILY_22_CODE = 32
SMP_OPT_SHED_DAILY_23_CODE = 33
SMP_OPT_SHED_DAILY_24_CODE = 34
SMP_OPT_SHED_DAILY_25_CODE = 35
SMP_OPT_SHED_DAILY_26_CODE = 36
SMP_OPT_SHED_DAILY_27_CODE = 37
SMP_OPT_SHED_DAILY_28_CODE = 38
SMP_OPT_SHED_DAILY_29_CODE = 39
SMP_OPT_SHED_DAILY_30_CODE = 40
SMP_OPT_SHED_DAILY_31_CODE = 41
SMP_OPT_SHED_DAILY_32_CODE = 42
SMP_OPT_SHED_WEEKLY_01_CODE = 43
SMP_OPT_SHED_WEEKLY_02_CODE = 44
SMP_OPT_SHED_WEEKLY_03_CODE = 45
SMP_OPT_SHED_WEEKLY_04_CODE = 46
SMP_OPT_SHED_WEEKLY_05_CODE = 47
SMP_OPT_SHED_WEEKLY_06_CODE = 48
SMP_OPT_SHED_WEEKLY_07_CODE = 49
SMP_OPT_SHED_WEEKLY_08_CODE = 50
SMP_OPT_SHED_WEEKLY_09_CODE = 51
SMP_OPT_SHED_WEEKLY_10_CODE = 52
SMP_OPT_SHED_WEEKLY_11_CODE = 53
SMP_OPT_SHED_WEEKLY_12_CODE = 54
SMP_OPT_SHED_WEEKLY_13_CODE = 55
SMP_OPT_SHED_WEEKLY_14_CODE = 56
SMP_OPT_SHED_WEEKLY_15_CODE = 57
SMP_OPT_SHED_WEEKLY_16_CODE = 58
SMP_OPT_SHED_WEEKLY_17_CODE = 59
SMP_OPT_SHED_WEEKLY_18_CODE = 60
SMP_OPT_SHED_WEEKLY_19_CODE = 61
SMP_OPT_SHED_WEEKLY_20_CODE = 62
SMP_OPT_SHED_WEEKLY_21_CODE = 63
SMP_OPT_SHED_WEEKLY_22_CODE = 64
SMP_OPT_SHED_WEEKLY_23_CODE = 65
SMP_OPT_SHED_WEEKLY_24_CODE = 66
SMP_OPT_SPEC_DAYS_ANNUAL_CODE = 67
SMP_OPT_SPEC_DAYS_1_CODE = 68
SMP_OPT_SPEC_DAYS_2_CODE = 69
SMP_OPT_NEXT_SEASONS_GROUP_CODE = 70
SMP_OPT_PERIOD_LAST_DAY_CODE = 71
SMP_OPT_SHED_POW_CTRL_01_CODE = 72
SMP_OPT_SHED_POW_CTRL_02_CODE = 73
SMP_OPT_SHED_POW_CTRL_03_CODE = 74
SMP_OPT_SHED_POW_CTRL_04_CODE = 75
SMP_OPT_SHED_POW_CTRL_05_CODE = 76
SMP_OPT_SHED_POW_CTRL_06_CODE = 77
SMP_OPT_SHED_POW_CTRL_07_CODE = 78
SMP_OPT_SHED_POW_CTRL_08_CODE = 79
SMP_OPT_SHED_POW_CTRL_09_CODE = 80
SMP_OPT_SHED_POW_CTRL_10_CODE = 81
SMP_OPT_SHED_POW_CTRL_11_CODE = 82
SMP_OPT_SHED_POW_CTRL_12_CODE = 83
SMP_OPT_POW_LIM_CTRL_CODE = 84
SMP_OPT_POW_LIM_MORNING_01_CODE = 85
SMP_OPT_POW_LIM_MORNING_02_CODE = 86
SMP_OPT_POW_LIM_MORNING_03_CODE = 87
SMP_OPT_POW_LIM_MORNING_04_CODE = 88
SMP_OPT_POW_LIM_MORNING_05_CODE = 89
SMP_OPT_POW_LIM_MORNING_06_CODE = 90
SMP_OPT_POW_LIM_MORNING_07_CODE = 91
SMP_OPT_POW_LIM_MORNING_08_CODE = 92
SMP_OPT_POW_LIM_MORNING_09_CODE = 93
SMP_OPT_POW_LIM_MORNING_10_CODE = 94
SMP_OPT_POW_LIM_MORNING_11_CODE = 95
SMP_OPT_POW_LIM_MORNING_12_CODE = 96
SMP_OPT_POW_LIM_EVENING_01_CODE = 97
SMP_OPT_POW_LIM_EVENING_02_CODE = 98
SMP_OPT_POW_LIM_EVENING_03_CODE = 99
SMP_OPT_POW_LIM_EVENING_04_CODE = 100
SMP_OPT_POW_LIM_EVENING_05_CODE = 101
SMP_OPT_POW_LIM_EVENING_06_CODE = 102
SMP_OPT_POW_LIM_EVENING_07_CODE = 103
SMP_OPT_POW_LIM_EVENING_08_CODE = 104
SMP_OPT_POW_LIM_EVENING_09_CODE = 105
SMP_OPT_POW_LIM_EVENING_10_CODE = 106
SMP_OPT_POW_LIM_EVENING_11_CODE = 107
SMP_OPT_POW_LIM_EVENING_12_CODE = 108
SMP_OPT_ENERGY_LIM_CTRL_CODE = 109
SMP_OPT_ENERGY_COST_T1_CODE = 110
SMP_OPT_ENERGY_COST_T2_CODE = 111
SMP_OPT_ENERGY_COST_T3_CODE = 112
SMP_OPT_ENERGY_COST_T4_CODE = 113
SMP_OPT_ENERGY_COST_T5_CODE = 114
SMP_OPT_ENERGY_COST_T6_CODE = 115
SMP_OPT_ENERGY_COST_T7_CODE = 116
SMP_OPT_ENERGY_COST_T8_CODE = 117
SMP_OPT_ENERGY_COST_DEFAULT_CODE = 118
SMP_OPT_ENERGY_LIM1_CODE = 119
SMP_OPT_ENERGY_LIM2_CODE = 120
SMP_OPT_ENERGY_LIM3_CODE = 121
SMP_OPT_VOLTAGE_LIM_MAX_CODE = 122
SMP_OPT_VOLTAGE_LIM_MIN_CODE = 123
SMP_OPT_VOLTAGE_LIM_HYST_CODE = 124
SMP_OPT_RELAY_LOAD_CTRL_CODE = 125
SMP_OPT_RELAY_ALARM_CTRL_CODE = 126
SMP_OPT_RELAY_ALARM_TIME_CODE = 127
SMP_OPT_BEEP_CTRL_CODE = 128
SMP_OPT_TEMPERATURE_LIM_CODE = 129
SMP_OPT_INTALARM_CTRL_CODE = 130
SMP_OPT_IND_NRG_CTRL_CODE = 131
SMP_OPT_IND_GROUPS_CODE = 132
SMP_OPT_IND_DATA_PAUSE_CODE = 133
SMP_OPT_IND_TIME_PAUSE_CODE = 134
SMP_OPT_IND_BATTERY_LIFE_CODE = 135
SMP_OPT_IND_DEPTHS_CODE = 136
SMP_OPT_MODE_DIFF_CURRENT_CODE = 137
SMP_OPT_CURRENTS_CTRL_LIM_CODE = 138
SMP_OPT_FREQUENCY_CTRL_LIM_CODE = 139
SMP_OPT_MISSCONSUMP_PERIOD_CODE = 140
SMP_OPT_MISSCONSUMP_LIM_CODE = 141
SMP_OPT_DATA_DIGITS_CODE = 142
SMP_OPT_PROFILE_INTERVAL_CODE = 143
SMP_OPT_NETPARAMS_CTRL_CODE = 144
SMP_OPT_CHANNEL_CODE = 145
SMP_OPT_OPER_CH_CTRL_CODE = 146
SMP_OPT_OPTOPORT_CTRL_CODE = 147
SMP_OPT_PW_READ_CODE = 148
SMP_OPT_PW_WRITE1_CODE = 149
SMP_OPT_PW_WRITE2_CODE = 150
SMP_OPT_PW_BLOCK_MODE_CODE = 151
SMP_OPT_ACTIONS_EVENT_01_CODE = 152
SMP_OPT_ACTIONS_EVENT_02_CODE = 153
SMP_OPT_ACTIONS_EVENT_03_CODE = 154
SMP_OPT_ACTIONS_EVENT_04_CODE = 155
SMP_OPT_ACTIONS_EVENT_05_CODE = 156
SMP_OPT_ACTIONS_EVENT_06_CODE = 157
SMP_OPT_ACTIONS_EVENT_07_CODE = 158
SMP_OPT_ACTIONS_EVENT_08_CODE = 159
SMP_OPT_ACTIONS_EVENT_09_CODE = 160
SMP_OPT_ACTIONS_EVENT_10_CODE = 161
SMP_OPT_ACTIONS_EVENT_11_CODE = 162
SMP_OPT_ACTIONS_EVENT_12_CODE = 163
SMP_OPT_ACTIONS_EVENT_13_CODE = 164
SMP_OPT_ACTIONS_EVENT_14_CODE = 165
SMP_OPT_ACTIONS_EVENT_15_CODE = 166
SMP_OPT_ACTIONS_EVENT_16_CODE = 167
SMP_OPT_ACTIONS_EVENT_17_CODE = 168
SMP_OPT_ACTIONS_EVENT_18_CODE = 169
SMP_OPT_ACTIONS_EVENT_19_CODE = 170
SMP_OPT_ACTIONS_EVENT_20_CODE = 171
SMP_OPT_ACTIONS_EVENT_21_CODE = 172
SMP_OPT_ACTIONS_EVENT_22_CODE = 173
SMP_OPT_ACTIONS_EVENT_23_CODE = 174
SMP_OPT_ACTIONS_EVENT_24_CODE = 175
SMP_OPT_SUBSCIBER_ADDRESS_CODE = 176
SMP_OPT_SUBSCRIBER_NUM_CODE = 177
SMP_OPT_SUBSCRIBER_MSG_TEXT_CODE = 178
SMP_OPT_TARIFF_PLAN_TEXT_CODE = 179
SMP_OPT_PRIORITY_T1_CODE = 180
SMP_OPT_PRIORITY_T2_CODE = 181
SMP_OPT_PRIORITY_T3_CODE = 182
SMP_OPT_PRIORITY_T4_CODE = 183
SMP_OPT_PRIORITY_T5_CODE = 184
SMP_OPT_PRIORITY_T6_CODE = 185
SMP_OPT_PRIORITY_T7_CODE = 186
SMP_OPT_PRIORITY_T8_CODE = 187
SMP_OPT_PRIORITY_G1_CODE = 188
SMP_OPT_PRIORITY_G2_CODE = 189
SMP_OPT_TELEMETRY_MODE_CODE = 190
SMP_OPT_CALIBR_TCOFVREF_CODE1 = 191
SMP_OPT_CALIBR_PLCONST_CODE1 = 192
SMP_OPT_CALIBR_MMODE_CODE1 = 193
SMP_OPT_CALIBR_LGAIN_CODE1 = 194
SMP_OPT_CALIBR_LPHI_CODE1 = 195
SMP_OPT_CALIBR_PSTARTTH_CODE1 = 196
SMP_OPT_CALIBR_PNOLTH_CODE1 = 197
SMP_OPT_CALIBR_QSTARTTH_CODE1 = 198
SMP_OPT_CALIBR_QNOLTH_CODE1 = 199
SMP_OPT_CALIBR_UGAIN_CODE1 = 200
SMP_OPT_CALIBR_IGAINL_CODE1 = 201
SMP_OPT_CALIBR_IOFFSETL_CODE1 = 202
SMP_OPT_CALIBR_POFFSETL_CODE1 = 203
SMP_OPT_CALIBR_QOFFSETL_CODE1 = 204
SMP_OPT_CALIBR_TCOFVREF_CODE2 = 205
SMP_OPT_CALIBR_PLCONST_CODE2 = 206
SMP_OPT_CALIBR_MMODE_CODE2 = 207
SMP_OPT_CALIBR_LGAIN_CODE2 = 208
SMP_OPT_CALIBR_LPHI_CODE2 = 209
SMP_OPT_CALIBR_PSTARTTH_CODE2 = 210
SMP_OPT_CALIBR_PNOLTH_CODE2 = 211
SMP_OPT_CALIBR_QSTARTTH_CODE2 = 212
SMP_OPT_CALIBR_QNOLTH_CODE2 = 213
SMP_OPT_CALIBR_UGAIN_CODE2 = 214
SMP_OPT_CALIBR_IGAINL_CODE2 = 215
SMP_OPT_CALIBR_IOFFSETL_CODE2 = 216
SMP_OPT_CALIBR_POFFSETL_CODE2 = 217
SMP_OPT_CALIBR_QOFFSETL_CODE2 = 218
SMP_OPT_SERIAL_NUM_CODE = 219
SMP_OPT_TELEMETRY_COEFF_CODE = 220
SMP_OPT_SERIAL_NUM_1_CODE = 221
SMP_OPT_SERIAL_NUM_2_CODE = 222
SMP_OPT_SERIAL_NUM_3_CODE = 223
SMP_OPT_SERIAL_NUM_4_CODE = 224
SMP_OPT_SUBSCRIBER_NUM_1_CODE = 225
SMP_OPT_SUBSCRIBER_NUM_2_CODE = 226
SMP_OPT_SUBSCRIBER_NUM_3_CODE = 227
SMP_OPT_SUBSCRIBER_NUM_4_CODE = 228
SMP_OPT_ENERGY_TYPE_1_CODE = 229
SMP_OPT_ENERGY_TYPE_2_CODE = 230
SMP_OPT_ENERGY_TYPE_3_CODE = 231
SMP_OPT_ENERGY_TYPE_4_CODE = 232
SMP_OPT_METER_TYPE_1_CODE = 233
SMP_OPT_METER_TYPE_2_CODE = 234
SMP_OPT_METER_TYPE_3_CODE = 235
SMP_OPT_METER_TYPE_4_CODE = 236
SMP_OPT_INPUT_FUNC_1_CODE = 237
SMP_OPT_INPUT_FUNC_2_CODE = 238
SMP_OPT_INPUT_FUNC_3_CODE = 239
SMP_OPT_INPUT_FUNC_4_CODE = 240
SMP_OPT_CONTACT_TYPE_1_CODE = 241
SMP_OPT_CONTACT_TYPE_2_CODE = 242
SMP_OPT_CONTACT_TYPE_3_CODE = 243
SMP_OPT_CONTACT_TYPE_4_CODE = 244
SMP_OPT_INPUT_TYPE_1_CODE = 245
SMP_OPT_INPUT_TYPE_2_CODE = 246
SMP_OPT_INPUT_TYPE_3_CODE = 247
SMP_OPT_INPUT_TYPE_4_CODE = 248
SMP_OPT_START_NAME_UNIT_1_CODE = 249
SMP_OPT_START_NAME_UNIT_2_CODE = 250
SMP_OPT_START_NAME_UNIT_3_CODE = 251
SMP_OPT_START_NAME_UNIT_4_CODE = 252
SMP_OPT_COEFF_NAME_UNIT_1_CODE = 253
SMP_OPT_COEFF_NAME_UNIT_2_CODE = 254
SMP_OPT_COEFF_NAME_UNIT_3_CODE = 255
SMP_OPT_COEFF_NAME_UNIT_4_CODE = 256
SMP_OPT_PROF_TYPE_1_CODE = 257
SMP_OPT_PROF_TYPE_2_CODE = 258
SMP_OPT_PROF_TYPE_3_CODE = 259
SMP_OPT_PROF_TYPE_4_CODE = 260
SMP_OPT_INPUT_ACTIVE_1_CODE = 261
SMP_OPT_INPUT_ACTIVE_2_CODE = 262
SMP_OPT_INPUT_ACTIVE_3_CODE = 263
SMP_OPT_INPUT_ACTIVE_4_CODE = 264
SMP_OPT_PERIOD_LAST_DAY_1_CODE = 265
SMP_OPT_PERIOD_LAST_DAY_2_CODE = 266
SMP_OPT_PERIOD_LAST_DAY_3_CODE = 267
SMP_OPT_PERIOD_LAST_DAY_4_CODE = 268
SMP_OPT_WAIT_TIME_CODE = 269
SMP_OPT_FLOOD_WND_CODE = 270
SMP_OPT_FLOOD_LTL_CODE = 271
SMP_OPT_SUBNET_CODE = 272
SMP_OPT_OPER_CHANNEL_CODE = 273
SMP_OPT_MODE_CODE = 274
SMP_OPT_REGISTRY_PERIOD_CODE = 275
SMP_OPT_REGISTRY_TO_CODE = 276
SMP_OPT_DATAOUT_PERIOD_CODE = 277
SMP_OPT_REGISTRY_DURATION_CODE = 278
SMP_OPT_DATAOUT_DEEP_CODE = 279
SMP_OPT_DATAOUT_LIST_CODE = 280
SMP_OPT_SMP_REQ_CODE = 281
SMP_OPT_DAY_LIMIT_BAT_CODE = 282
SMP_OPT_CURRENT_LIM_MAX_CODE = 283
SMP_OPT_CURRENT_LIM_MIN_CODE = 284
SMP_OPT_CURRENT_LIM_HYST_CODE = 285
SMP_OPT_LIGHT_ON_CODE = 286
SMP_OPT_TARIF_ENE_MODE_CODE = 287
SMP_OPT_NETPARAMS_LIST_CODE = 288
SMP_OPT_3PH_PROFILE_PARAM0_CODE = 289
SMP_OPT_3PH_PROFILE_PARAM1_CODE = 290
SMP_OPT_3PH_PROFILE_PARAM2_CODE = 291
SMP_OPT_3PH_PROFILE_PARAM3_CODE = 292
SMP_OPT_3PH_PROFILE_PARAM4_CODE = 293
SMP_OPT_3PH_PROFILE_PARAM5_CODE = 294
SMP_OPT_3PH_PROFILE_PARAM6_CODE = 295
SMP_OPT_3PH_PROFILE_PARAM7_CODE = 296
SMP_OPT_3PH_PROFILE_PARAM8_CODE = 297
SMP_OPT_3PH_PROFILE_PARAM9_CODE = 298
SMP_OPT_3PH_PROFILE_PARAM10_CODE = 299
SMP_OPT_3PH_PROFILE_PARAM11_CODE = 300
SMP_OPT_3PH_PROFILE_PARAM12_CODE = 301
SMP_OPT_3PH_PROFILE_PARAM13_CODE = 302
SMP_OPT_3PH_PROFILE_PARAM14_CODE = 303
SMP_OPT_3PH_PROFILE_PARAM15_CODE = 304
SMP_OPT_POW_LIM_MIDDAY_01_CODE = 305
SMP_OPT_POW_LIM_MIDDAY_02_CODE = 306
SMP_OPT_POW_LIM_MIDDAY_03_CODE = 307
SMP_OPT_POW_LIM_MIDDAY_04_CODE = 308
SMP_OPT_POW_LIM_MIDDAY_05_CODE = 309
SMP_OPT_POW_LIM_MIDDAY_06_CODE = 310
SMP_OPT_POW_LIM_MIDDAY_07_CODE = 311
SMP_OPT_POW_LIM_MIDDAY_08_CODE = 312
SMP_OPT_POW_LIM_MIDDAY_09_CODE = 313
SMP_OPT_POW_LIM_MIDDAY_10_CODE = 314
SMP_OPT_POW_LIM_MIDDAY_11_CODE = 315
SMP_OPT_POW_LIM_MIDDAY_12_CODE = 316
SMP_OPT_SHED_POW_3ZONE_CTRL_01_CODE = 317
SMP_OPT_SHED_POW_3ZONE_CTRL_02_CODE = 318
SMP_OPT_SHED_POW_3ZONE_CTRL_03_CODE = 319
SMP_OPT_SHED_POW_3ZONE_CTRL_04_CODE = 320
SMP_OPT_SHED_POW_3ZONE_CTRL_05_CODE = 321
SMP_OPT_SHED_POW_3ZONE_CTRL_06_CODE = 322
SMP_OPT_SHED_POW_3ZONE_CTRL_07_CODE = 323
SMP_OPT_SHED_POW_3ZONE_CTRL_08_CODE = 324
SMP_OPT_SHED_POW_3ZONE_CTRL_09_CODE = 325
SMP_OPT_SHED_POW_3ZONE_CTRL_10_CODE = 326
SMP_OPT_SHED_POW_3ZONE_CTRL_11_CODE = 327
SMP_OPT_SHED_POW_3ZONE_CTRL_12_CODE = 328
SMP_OPT_ACTIONS_EVENT_ORDER_FAZ_CODE = 329
SMP_OPT_ACTIONS_EVENT_ERR_FAZ_CODE = 330
SMP_OPT_ACTIONS_RADIOFIELD_CODE = 331
SMP_OPT_KOEF_TRANSF_CURRENT_CODE = 332
SMP_OPT_IND_GROUPS_CODE_2 = 333
SMP_OPT_DEVICE_VARIATION_CODE = 334
SMP_OPT_KOEF_TRANSF_VOLTAGE_CODE = 335
SMP_OPT_TRANSFORM_MODE = 336
SMP_OPT_ACTIONS_EVENT_INPUT1 = 337
SMP_OPT_ACTIONS_EVENT_INPUT2 = 338
SMP_OPT_TEMPERATURE_TRESHOLD = 339
SMP_OPT_LIGHT_ON_LENGTH_CODE = 340

SMP_OPT_BASE_VOLTAGE = 342
SMP_OPT_VOLTAGE_LIM_MAX_PERCENT_CODE = 343
SMP_OPT_VOLTAGE_LIM_MIN_PERCENT_CODE = 344
SMP_OPT_DELTA_I_N_LIM_CODE = 345
SMP_OPT_EVENT_GROUPS_1_CODE = 346
SMP_OPT_INDICATION_DEFFERED = 347
SMP_OPT_TEMPERATURE_LIM_MIN_CODE = 348
SMP_OPT_TEMPERATURE_LIM_MAX_CODE = 349
SMP_OPT_ACTIONS_EVENT_VAR_MAGNET = 350
SMP_OPT_ACTIONS_EVENT_I_N_DIFF = 351
SMP_OPT_ACTIONS_EVENT_TEMPERATURE1 = 352
SMP_OPT_ACTIONS_EVENT_TEMPERATURE2 = 353
SMP_OPT_ACTIONS_EVENT_LIMIT_I_MIN = 354
SMP_OPT_ACTIONS_EVENT_LIMIT_I_MAX = 355
SMP_OPT_ENERGY_COUNT_ALG_S = 356
SMP_OPT_ENERGY_COUNT_ALG_P = 357
SMP_OPT_ENERGY_COUNT_ALG_Q = 358
SMP_OPT_RELAY_LOAD_START_STATE_CODE = 359
SMP_OPT_RELAY_ALARM_START_STATE_CODE = 360
SMP_OPT_RELAY_LOAD_RETURN_CODE = 361
SMP_OPT_RELAY_ALARM_RETURN_CODE = 362
SMP_OPT_RELAY_LOAD_DELAY_CODE = 363
SMP_OPT_RELAY_ALARM_DELAY_CODE = 364
SMP_OPT_IMPULSE_INPUT_1 = 365
SMP_OPT_IMPULSE_INPUT_2 = 366
SMP_OPT_FORCED_TARIF_INDICATION = 367
SMP_OPT_CUR_POW_CONTROL = 368
SMP_OPT_CUR_POW_LIMIT = 369
SMP_OPT_MAGPARAMS_CODE = 371

SMP_OPT_RFPARAMS_CODE = 373
SMP_OPT_PROFILE_PARAM_1_CODE = 374
SMP_OPT_PROFILE_PARAM_2_CODE = 375
SMP_OPT_PROFILE_PARAM_3_CODE = 376
SMP_OPT_PROFILE_PARAM_4_CODE = 377
SMP_OPT_PROFILE_PARAM_5_CODE = 378
SMP_OPT_PROFILE_PARAM_6_CODE = 379
SMP_OPT_PROFILE_PARAM_7_CODE = 380
SMP_OPT_PROFILE_PARAM_8_CODE = 381
SMP_OPT_PROFILE_PARAM_9_CODE = 382
SMP_OPT_PROFILE_PARAM_10_CODE = 383
SMP_OPT_PROFILE_PARAM_11_CODE = 384
SMP_OPT_PROFILE_PARAM_12_CODE = 385
SMP_OPT_PROFILE_PARAM_13_CODE = 386
SMP_OPT_PROFILE_PARAM_14_CODE = 387
SMP_OPT_PROFILE_PARAM_15_CODE = 388
SMP_OPT_PROFILE_PARAM_16_CODE = 389
SMP_OPT_PROFILE_PARAM_17_CODE = 390
SMP_OPT_PROFILE_PARAM_18_CODE = 391
SMP_OPT_PROFILE_PARAM_19_CODE = 392
SMP_OPT_PROFILE_PARAM_20_CODE = 393
SMP_OPT_PROFILE_PARAM_21_CODE = 394
SMP_OPT_PROFILE_PARAM_22_CODE = 395
SMP_OPT_PROFILE_PARAM_23_CODE = 396
SMP_OPT_PROFILE_PARAM_24_CODE = 397

SMP_OPT_COMM_EXTENSION_TYPE = 399
SMP_OPT_GSM_APN_NAME = 400
SMP_OPT_GSM_APN_LOGIN = 401
SMP_OPT_GSM_APN_PASSW = 402
SMP_OPT_GSM_PIN = 403
SMP_OPT_GSM_MODE = 404
SMP_OPT_DSTP_MODE = 405
SMP_OPT_TAU_AVERAGE_POW_LIMP_CODE = 406
SMP_OPT_TAU_RELE_OFF_LIMP_CODE = 407
SMP_OPT_OPTDEBUG_CODE = 408
SMP_OPT_RADIO_QUARTZ_FREQUENCY = 409
SMP_OPT_RS485_PREF = 410
SMP_OPT_RS485_RETRAN = 411
SMP_OPT_LIGHT_MODE = 412
SMP_OPT_LIGHT_CORRECTION = 413
SMP_OPT_LIGHT_TIMEZONE = 414
SMP_OPT_LIGHT_TIMESHIFT = 415
SMP_OPT_LIGHT_COORDINATES = 416
SMP_OPT_LIGHT_BREAKS = 417
SMP_OPT_LIGHT_CHECKTIME = 418
SMP_OPT_MODE_TARIF_SUM = 419
SMP_OPT_GSM_PORT = 420
SMP_OPT_DEFAULT_TARIF_CODE = 421
SMP_OPT_CURRENT_OFF = 422
SMP_OPT_RS485_2_PREF = 423
SMP_OPT_LIGHT_LEVEL_CODE = 424
SMP_OPT_SPLIT_MAC = 425
SMP_OPT_SPLIT_CONNECT = 426
SMP_OPT_SPLIT_CONFIG = 427

SMP_OPT_SERVER_LOGIN = 434
SMP_OPT_SERVER_PASSWORD = 435
SMP_OPT_SERVER_ADDRESS = 436

#коды INFO
SMP_INFO_AUTHORIZATION1_CODE = 1
SMP_INFO_AUTHORIZATION2_CODE = 2
SMP_INFO_VERSION_CODE = 3
SMP_INFO_SERIAL_NUM_CODE = 4
SMP_INFO_ID_CODE = 5
SMP_INFO_SUBNET_CODE = 6
SMP_INFO_ACTUAL_TARIFF_CODE = 7
SMP_INFO_ACTUAL_DSHED_CODE = 8
SMP_INFO_ACTUAL_WSHED_CODE = 9
SMP_INFO_ACTUAL_WSHED_GROUP_CODE = 10
SMP_INFO_CURRENT_ACCOUNT_CODE = 11
SMP_INFO_CREDIT_BALANCE_CODE = 12
SMP_INFO_SOCIAL_LIM_BALANCE_CODE = 13
SMP_INFO_PREPAYMENT_CODE = 14
SMP_INFO_RELAY_LOAD_STATE_CODE = 15
SMP_INFO_RELAY_ALARM_STATE_CODE = 16
SMP_INFO_CONST_K_CODE = 17
SMP_INFO_CONST_CHECK_RECALC_CODE = 18
SMP_INFO_CRC_CALIBR_CONST_L_CODE = 19
SMP_INFO_CRC_CALIBR_CONST_N_CODE = 20
SMP_INFO_CRC_CONFIG_CODE = 21
SMP_INFO_CRC_METROLOGY_CODE = 22
SMP_INFO_STATUS_CODE = 23
SMP_INFO_AUTHORIZATION_CODE = 24
SMP_INFO_LAST_25HOUR_DATE_CODE = 25
SMP_INFO_CRC_CONFIG_JOB_FULL_CODE = 26
SMP_INFO_CRC_CONFIG_JOB_PARTIAL_CODE = 27
SMP_INFO_CRC_CONFIG_FON_FULL_CODE = 28
SMP_INFO_CRC_CONFIG_FON_PARTIAL_CODE = 29
SMP_INFO_SUMMATOR_STATUS_CODE = 30
SMP_INFO_LAST_TRANSMITT_DATE_CODE = 31
SMP_INFO_LAST_CONFIRM_DATE_CODE = 32
SMP_INFO_SUMMATOR_MODE_CODE = 33
SMP_INFO_SIGNAL_LEVEL_CODE = 34
SMP_INFO_ACTUAL_POW_CTRL_SHED_CODE = 35
SMP_INFO_INTERFACE_MESSAGE_CODE = 36
SMP_INFO_WAS_CALIBRATION_CODE = 37
SMP_INFO_ACTUAL_TARIFF_AN_CODE = 38
SMP_INFO_ACTUAL_DSHED_AN_CODE = 39
SMP_INFO_ACTUAL_WSHED_AN_CODE = 40
SMP_INFO_ACTUAL_WSHED_GROUP_AN_CODE = 41
SMP_INFO_CPU_SERIAL_NUMBER_CODE = 42
SMP_INFO_ACTUAL_TARIFF_RP_CODE = 43
SMP_INFO_ACTUAL_DSHED_RP_CODE = 44
SMP_INFO_ACTUAL_WSHED_RP_CODE = 45
SMP_INFO_ACTUAL_WSHED_GROUP_RP_CODE = 46
SMP_INFO_ACTUAL_TARIFF_RN_CODE = 47
SMP_INFO_ACTUAL_DSHED_RN_CODE = 48
SMP_INFO_ACTUAL_WSHED_RN_CODE = 49
SMP_INFO_ACTUAL_WSHED_GROUP_RN_CODE = 50
SMP_INFO_DEVICE_VARIATION_CODE = 51
SMP_INFO_ID_1 = 52
SMP_INFO_ID_2 = 53
SMP_INFO_ID_3 = 54
SMP_INFO_ID_4 = 55
SMP_INFO_ID_5 = 56

SMP_INFO_ELECTROENERGY_QUALITY = 58
SMP_INFO_DEVICE_CONSUMING = 59

SMP_INFO_SENSOR_1_STATE = 61
SMP_INFO_SENSOR_2_STATE = 62
SMP_INFO_SENSOR_3_STATE = 63
SMP_INFO_SENSOR_4_STATE = 64
SMP_INFO_GPRS = 65

SMP_INFO_LIGHT_ASTRO = 67

SMP_INFO_CHECK_CRC_CONFIG_CODE = 71
SMP_INFO_EXTENSION_VERSION = 73
SMP_INFO_LIGHT_ASTRO_CURRENT = 74
SMP_INFO_EXTENSION_GSM_IMEI = 76
SMP_INFO_STATE_ERROR_CODE = 77
SMP_INFO_DATA_MODE = 78
SMP_INFO_TARIF_USING = 79
SMP_INFO_EXTENSION_DESCRIPTION = 80

#коды ACTION
SMP_ACTION_AUTHORIZE_CODE = 1
SMP_ACTION_COPYCONFIG_FON_TO_JOB_CODE = 2
SMP_ACTION_COPYCONFIG_JOB_TO_FON_CODE = 3
SMP_ACTION_CHANGECONFIG_JOB_FON_CODE = 4
SMP_ACTION_SETCONFIG_FACTORY_CODE = 5
SMP_ACTION_MESSAGE_CODE = 6
SMP_ACTION_GET_NEXT_PACKET_CODE = 7
SMP_ACTION_ALARM_FLAG_CLR_CODE = 8
SMP_ACTION_SET_TARIF_CODE = 9
SMP_ACTION_SET_RELAY_LOAD_CODE = 10
SMP_ACTION_SET_RELAY_ALARM_CODE = 11
SMP_ACTION_SOUND_ALARM_ON_CODE = 12
SMP_ACTION_SOUND_ALARM_OFF_CODE = 13
SMP_ACTION_FIX_DATA_CODE = 14
SMP_ACTION_CORRECT_TIME_CODE = 15
SMP_ACTION_CALIBRATION_SMALLPOWER = 16
SMP_ACTION_CALIBRATION_LGAIN1 = 17
SMP_ACTION_CALIBRATION_LGAIN05 = 18
SMP_ACTION_CALIBRATION_POFFSET1 = 19
SMP_ACTION_CALIBRATION_POFFSET2 = 20
SMP_ACTION_CALIBRATION_UGAIN = 21
SMP_ACTION_CALIBRATION_IGAIN = 22
SMP_ACTION_CALIBRATION_EXIT_CODE = 23
SMP_ACTION_APPROVE_CODE = 24
SMP_ACTION_CLEAR_FULL_EEPROM_CODE = 25
SMP_ACTION_CLEAR_TARIF_COUNTER_CODE = 26
SMP_ACTION_CLEAR_PROFILE_INTERVAL_CODE = 27
SMP_ACTION_CLEAR_MAX_POW_CODE = 28
SMP_ACTION_RESET_PASSWORD_CODE = 29
SMP_ACTION_RESET_CNT_MAGNET_CODE = 30
SMP_ACTION_RESET_CNT_VCC_ABSENT_CODE = 31
SMP_ACTION_RESET_CNT_VCC_LOW_CODE = 32
SMP_ACTION_RESET_CNT_VCC_HI_CODE = 33
SMP_ACTION_RESET_CNT_OVER_LIMP_CODE = 34
SMP_ACTION_RESET_CNT_OVER_I_N_CODE = 35
SMP_ACTION_RESET_CNT_OVER_FREQ_CODE = 36
SMP_ACTION_RESET_CNT_OVER_I_L_CODE = 37
SMP_ACTION_RESET_CNT_RESYNC_TIME_CODE = 38
SMP_ACTION_RESET_AUTHORIZE_CODE = 39
SMP_ACTION_SET_MODE_CODE = 40
SMP_ACTION_SYNC_CODE = 41
SMP_ACTION_SYNC_WTIME_CODE = 42
SMP_ACTION_SET_FACTORY_NUM_CODE = 43
SMP_ACTION_SETUP_DEVICE_CODE = 44
SMP_ACTION_SET_BEACON_CODE = 45
SMP_ACTION_SECOND_OUTPUT_CODE = 46
SMP_ACTION_CLEAR_EVLOG_CODE = 47
SMP_ACTION_PAYMENT = 48
SMP_ACTION_RESET_BALANCE = 49
SMP_ACTION_CALIBR_3PH_STAGE1 = 50
SMP_ACTION_CALIBR_3PH_STAGE2 = 51
SMP_ACTION_CALIBR_3PH_STAGE3 = 52
SMP_ACTION_CALIBR_3PH_STAGE4 = 53
SMP_ACTION_CALIBR_3PH_STAGE5 = 54
SMP_ACTION_CALIBR_3PH_EXIT = 55
SMP_ACTION_RESET_CNT_ERR_ORDER_FAZ_CODE = 56
SMP_ACTION_RESET_CNT_OVER_RADIO_CODE = 57
SMP_ACTION_RESET_CNT_VAR_MAGNET_CODE = 58
SMP_ACTION_CALIBRATION_SET = 59
SMP_ACTION_CALIBRATION_CORRECTION_VAL = 60
SMP_ACTION_CALIBRATION_CORRECTION_ERR = 61

SMP_ACTION_RESET_TARIF_INDICATION = 63
SMP_ACTION_WRITE_ID1 = 64
SMP_ACTION_WRITE_ID2 = 65
SMP_ACTION_WRITE_ID3 = 66
SMP_ACTION_WRITE_ID4 = 67
SMP_ACTION_WRITE_ID5 = 68
SMP_ACTION_T10_CONTROL = 69
SMP_ACTION_T11_CONTROL = 70
SMP_ACTION_SEAL = 71
SMP_ACTION_DEFERRED_SEAL = 72
SMP_ACTION_RESET_INPUT_COUNTER1 = 73
SMP_ACTION_RESET_INPUT_COUNTER2 = 74
SMP_ACTION_TECH_TIME_OFF = 75
SMP_ACTION_CALIBRATION_STAGE = 76
SMP_ACTION_TAMPEROFF_MODE = 77
SMP_ACTION_RESET_CNT_ALLVIOLATIONS_CODE = 80
SMP_ACTION_SELF_CHECK = 81
SMP_ACTION_CONFIRM_CRC = 82


GET_DATA_SINGLE = 1
GET_DATA_MULTIPLE = 2
GET_OPTION = 4
SET_OPTION = 5
GET_INFO = 6
PERFORM_ACTION = 7
GET_OPTION1 = 8
GETSET = 9
GET_DATA_SINGLE_EX = 10
GET_DATA_MULTIPLE_EX = 11

UNKNOWN = ('UNKNOWN',)
NUM = ('NUM',)
INT = ('INT',)
STRING = ('STRING',)
DATETIME = ('DATETIME',)


def DOUBLE(descr):
    return ('DOUBLE', int(descr))


def BITBUF(descr):
    return ('BITBUF', descr)



from .types import *
from .codes import *

data_single = {
    SMP_SDATA_TEMPERATURE_CODE: INT,
    SMP_SDATA_BATTERY_CODE: DOUBLE(2),
    SMP_SDATA_DATETIME_CODE: DATETIME,
    SMP_SDATA_COUNT_CH1_CODE: NUM,
    SMP_SDATA_COUNT_CH2_CODE: NUM,
}

info = {
    SMP_INFO_VERSION_CODE: BITBUF([8,8,8,8,8,8]),
    SMP_INFO_ID_CODE: NUM,
    SMP_INFO_EXTENSION_VERSION: STRING,
    SMP_INFO_EXTENSION_GSM_IMEI: STRING,
    SMP_INFO_EXTENSION_DESCRIPTION: STRING,
}

option = {
    SMP_OPT_TIME_DATE_CODE: DATETIME,
    SMP_OPT_GSM_APN_NAME: STRING,
    SMP_OPT_SERVER_LOGIN: STRING,
    SMP_OPT_SERVER_PASSWORD: STRING,
    SMP_OPT_SERVER_ADDRESS: STRING,
}

action = {
    SMP_ACTION_CONFIRM_CRC: NUM,
}



import datetime

from . import codes
from .code_types import info, action, option, data_single
from .core import SmpException, pack_bitbuf, unpack_bitbuf
from .types import *
from .value import *


def inspect_code_names():
    res = {
        'SDATA': {},
        'MDATA': {},
        'INFO': {},
        'OPT': {},
        'ACTION': {},
    }

    for name in dir(codes):
        parts = name.split('_')
        if len(parts) > 2 and parts[0] == 'SMP':
            res[parts[1]][getattr(codes, name)] = name

    return res


code_names = inspect_code_names()


class Command:
    type = UNKNOWN
    value = None

    def __init__(self, code):
        self.code = code

    @property
    def name(self):
        return 'SMP_UNKNOWN_CODE'

    @staticmethod
    def make_base_value(type, value):
        typename = type[0]
        if typename == 'NUM':
            res = Value(int(value))
        elif typename == 'INT':
            res = Value(int(value))
        elif typename == 'STRING':
            res = Value(str(value))
        elif typename == 'DATETIME':
            if type(value) is datetime.datetime:
                res = Value(value)
            else:
                raise SmpException()
        elif typename == 'DOUBLE':
            res = Value(float(value))
        elif typename == 'BITBUF':
            res = Value(value)
        else:
            raise SmpException()

        return res

    @staticmethod
    def write_base_value(type, packet, value):
        typename = type[0]
        if typename == 'NUM':
            packet.write_num(value.value)
        elif typename == 'INT':
            packet.write_int(value.value)
        elif typename == 'STRING':
            packet.write_string(value.value)
        elif typename == 'DATETIME':
            packet.write_datetime(value.value)
        elif typename == 'DOUBLE':
            packet.write_num(int(value.value * 10 ** type[1]))
        elif typename == 'BITBUF':
            packet.write_buffer(pack_bitbuf(value.value, type[1]))
        else:
            raise SmpException()

    @staticmethod
    def read_base_value(type, packet):
        typename = type[0]
        if typename == 'NUM':
            res = Value(packet.read_num())
        elif typename == 'INT':
            res = Value(packet.read_int())
        elif typename == 'STRING':
            res = Value(packet.read_string())
        elif typename == 'DATETIME':
            res = Value(packet.read_datetime())
        elif typename == 'DOUBLE':
            res = Value(packet.read_num() / 10 ** type[1])
        elif typename == 'BITBUF':
            res = Value(unpack_bitbuf(packet.read_buffer()[0], type[1]))
        else:
            raise SmpException()

        return res

    def require_dff(self):
        return self.code > 255

    def decode_param(self, packet):
        pass

    def encode_param(self, packet):
        pass

    def decode_value(self, packet):
        pass

    def encode_value(self, packet):
        pass


class CommandAction(Command):
    def __init__(self, code, value):
        super().__init__(code)
        self.type = action.get(code, UNKNOWN)
        self.value = self.make_base_value(self.type, value)

    def code_name(self):
        return code_names['ACTION'].get(self.code, 'SMP_UNKNOWN_CODE')

    def decode_value(self, packet):
        pass

    def encode_param(self, packet):
        self.write_base_value(self.type, packet, self.value)


class CommandInfo(Command):
    def __init__(self, code):
        super().__init__(code)
        self.type = info.get(code, UNKNOWN)

    def __eq__(self, other):
        return type(self) == type(other) and self.code == other.code

    @property
    def name(self):
        return code_names['INFO'].get(self.code, 'SMP_UNKNOWN_CODE')

    def decode_value(self, packet):
        return self.read_base_value(self.type, packet)


class CommandGetOption(Command):
    def __init__(self, code):
        super().__init__(code)
        self.type = option.get(code, UNKNOWN)

    def __eq__(self, other):
        return type(self) == type(other) and self.code == other.code

    @property
    def name(self):
        return code_names['OPT'].get(self.code, 'SMP_UNKNOWN_CODE')

    def decode_value(self, packet):
        return self.read_base_value(self.type, packet)


class CommandSetOption(Command):
    def __init__(self, code, value):
        super().__init__(code)
        self.type = option.get(code, UNKNOWN)
        self.value = self.make_base_value(self.type, value)

    def code_name(self):
        return code_names['OPT'].get(self.code, 'SMP_UNKNOWN_CODE')

    def decode_value(self, packet):
        pass

    def encode_param(self, packet):
        self.write_base_value(self.type, packet, self.value)


class CommandDataSingle(Command):
    def __init__(self, code, mask = 0):
        super().__init__(code)
        self.type = data_single.get(code, UNKNOWN)
        self.mask = mask

    def __eq__(self, other):
        return type(self) == type(other) and self.code == other.code and self.mask == other.mask

    @property
    def name(self):
        return code_names['SDATA'].get(self.code, 'SMP_UNKNOWN_CODE')

    @property
    def has_mask(self):
        self.type[0] == 'MASK'

    def decode_value(self, packet):
        if self.has_mask:
            raise SmpException()
        else:
            return self.read_base_value(self.type, packet)

    def decode_param(self, packet):
        pass

    def encode_param(self, packet):
        pass


class CommandDataMultiple(Command):
    def __init__(self, code, mask = 0, start = 0, end = 0):
        super().__init__(code)

    @property
    def name(self):
        return code_names['MDATA'].get(self.code, 'SMP_UNKNOWN_CODE')


class CommandDataSingleEx(Command):
    def __init__(self, code, mask = 0, start = 0, end = 0):
        super().__init__(code)


class CommandDataMultipleEx(Command):
    def __init__(self, code, mask = 0, start = 0, end = 0):
        super().__init__(code)



class ValueError(Exception):
    pass


class Value:
    _error = None
    _value = None

    def __init__(self, value = None):
        self._value = value

    @property
    def error(self):
        return self._error

    @property
    def value(self):
        if self._error is not None:
            raise ValueError()
        return self._value
